package fi.jyu.mit.ohj2;
import java.io.*;
import java.util.*;

/**
 * Luokka avustusten tulostamiseksi.  Avustustiedoston muoto:
 * <pre>
 * [SISLLYS] - aina etsitn sisllyst tll aihe- (topic) nimell.
 * Eka aihe - kerrotaan ekasta aiheesta
 * Toka aihe - kerrotaan tokasta aiheesta
 *
 * [Eka aihe]
 * Ekassa aiheessa voidaan kuvata mit vaan ekaan aiheeseen liittyv
 * ja miksei muutakin.
 *
 * [Toka aihe] - aiheen otsikkorivill saa olla kommentti
 * Tokassa aiheessa sitten tokan aiheen sisllst.
 * Tietysti aiheita voidaan kirjoittaa niin monta kuin halutaan
 * eik niiden kaikkien tarvitse olla sisllysluettelossa.
 * Jos on kamalan pitk aihe, jonka tulostus halutaan keskeytt,
 * voidaan avustustiedoston rivi aloittaa
 * #
 * risuaitamerkill, joka pysytt tulostuksen.
 * muutenkin tulostetaan vain korkeintaan 23 rivi kerrallaan.
 * ; puolipisteell alkava rivi on kommenttia ja sit ei tulosteta
 * </pre>
 * Kyttesimerkkej:<br>
 * Selaillaan koko avustusta.  Tulostetaan ensin sisllysluettelo
 * <pre>
 *   try {
 *     Help h = new Help("kerho.hlp");
 *     h.browse();
 *   }
 *   catch (IOException ioe) {
 *     System.err.println(ioe);
 *   }
 * </pre>
 * Tulostetaan valitun aiheen kohdalta:
 * <pre>
 *   h.printMatchingTopics("Li*");  // tulostaa kaikki Li-alkavat aiheet
 *   h.printTopic("Lisys");        // tulostaa aiheen Lisys
 *   h.helpTopic("Li*");            // tulostaa kaikki Li-alkavat aiheet
 * </pre>
 * @author Vesa Lappalainen, Markku Vire
 * @version 1.0, 24.03.2003
 */
public class Help {
  private static final char COMMENT = ';';
  private static final char TOPIC_START = '[';
  private static final char TOPIC_END = ']';
  private static final char HELP_PAUSE = '#';
  private static final char QUIT_CHAR = 'q';
  private static final int  DEFAULT_LINES = 23;
  private static final String INDEX = "SISLLYS";

  private HashMap topics = new HashMap();
  private int lines = DEFAULT_LINES;
  private int printedLineCounter = 0;
  private BufferedReader in = new BufferedReader(new InputStreamReader(System.in));

  /**
   * @return kuinka monta rivi tulostetaan korkeintaan pyshtymtt
   */
  public int getLines() { return lines; }

  /**
   * @param lines pyshtymtt tulostettavien rivien lukumr
   */
  public void setLines(int lines) {
    if ( lines > 0 )
      this.lines = lines;
  }

  /**
   * @param input tutkittava rivi
   * @return alkaako rivi lopetusmerkill
   */
  private static boolean quit(String input) {
    return firstIs(input, QUIT_CHAR);
  }

  /**
   * @param s tutkittava rivi
   * @param c merkki jota etsitn
   * @return onko c rivin s ensimminen merkki (true) vai ei (false)
   */
  private static boolean firstIs(String s, char c) {
    if ( s == null || s.length() == 0 )
      return false;

    return (Character.toUpperCase(s.charAt(0)) == Character.toUpperCase(c));
  }

  /**
   * Pyhtyy odottamaan reurnin panamista jos on tulostettuja rivej
   * edellisen pyshtymisen jlkeen.
   * @return onko painettu postumista (true) vai ei (false)
   */
  private boolean helpStop() {
    String input = null;

    if (printedLineCounter != 0) {
      printedLineCounter = 0;
      System.out.print("Jatka ENTER > ");
      try {
        input = in.readLine();
      } catch (IOException ioe) {
        System.out.println(ioe);
      }
    }

    return quit(input);
  }

  /**
   * Listn uusi aihe-otsikko avustukseen.
   * Tmn jlkeen rivej voi list otsikon alle
   * <pre>
   * Collection topic = h.addTopic("Uusi aihe");
   * topic.add("Eka rivi");
   * topic.add("Toka rivi");
   * </pre>
   * Mikli aihe on jo olemassa, palautetaan viite vanhaan aiheeseen.
   * @param topic listtvn aiheen otsikko
   * @return tietorakenne, johon voi list rivej aiheen alle
   */
  public Collection addTopic(String topic) {
    topic = topic.toUpperCase();
    Collection newtopic = (Collection)topics.get(topic.toUpperCase());
    if ( newtopic != null ) return newtopic;

    newtopic = new ArrayList();
    topics.put(topic,newtopic);
    return newtopic;
  }

  /**
   * Listn yksi rivi avustukseen aiheen topic alle.  Mikli aihetta
   * ei viel ole, se luodaan.
   * @param topic listtvn aiheen otsikko
   * @param line listtv rivi
   */
  public void addLine(String topic, String line) {
    Collection topicLines = addTopic(topic);
    topicLines.add(line);
  }

  /**
   * Alustetaan tyhj avustus, johon voi list aiheita
   * metodeilla: addTopic, addLine, readFile.
   */
  public Help()  {  }

  /**
   * Lukee avustuksen tiedostosta.  Voidaan kutsua useita kertoja
   * jolloin voidaan yhdist monia avustustiedostoja.
   *
   * Tiedot kertn map-tauluun, jossa aiheiden mukaiset merkkijonot
   * avaimina (isoksi muutettuna).  Aiheen alaiset rivit on
   * sitten vektorina "avaimen oliona".
   * @param fileName tiedosto,josta avustukset luetaan
   * @throws IOException jos jokin menee pieleen tiedoston lukemisessa
   */
  public void readFile(String fileName) throws IOException {
    BufferedReader in = new BufferedReader(new FileReader(fileName));
    Collection topic = null;
    String line;
    try {

      while ( (line = in.readLine()) != null ) {
        int comment = line.indexOf(COMMENT);

        if ( comment >= 0 ) line = line.substring(0, comment);

        if ( firstIs(line, TOPIC_START) ) {
          int topicEnd = line.indexOf(TOPIC_END);

          if (topicEnd >= 0) {
            String topicName = line.substring(1, topicEnd);

            topic = addTopic(topicName);
          }
        } else {
          if (topic != null)
            topic.add(line);
        }
      }
    } finally {
      in.close();
    }
  }

  /**
   * Alustaa avustuksen lukemalla avustukset tiedostosta.
   * Tiedot kertn map-tauluun, jossa aiheiden mukaiset merkkijonot
   * avaimina (isoksi muutettuna).  Aiheen alaiset rivit on
   * sitten vektorina "avaimen oliona".
   * @param fileName tiedosto,josta avustukset luetaan
   * @throws IOException jos jokin menee pieleen tiedoston lukemisessa
   */
  public Help(String fileName) throws IOException {
    readFile(fileName);
  }

  /**
   * Tulostaa seuraavan rivin Help-teksti.  Jos rivimr ylittyy,
   * niin pyshtyy.
   * @param text tulostettava rivi
   * @return painettiinko poistumista (true) vai ei (false) tulostuksen aikana
   */
  private boolean helpPrint(String text) {
    if ( firstIs(text, HELP_PAUSE) ) return helpStop();

    System.out.println(text);
    printedLineCounter++;

    if ( printedLineCounter >= lines ) return helpStop();

    return false;
  }

  /**
   * Tulostaa valitun lohkon avustuksesta.
   * @param topic tulostettava lohko.  Ei saa sislt jokereita
   * @return painettiinko poistumista (true) vai ei (false) tulostuksen aikana
   */
  public boolean printTopic(String topic) {
    Collection topicText = (Collection)topics.get(topic.toUpperCase());

    if ( topicText == null )
      return helpPrint("Aihetta " + topic + " ei lydy");

    for (Iterator i = topicText.iterator(); i.hasNext(); )
      if ( helpPrint((String)i.next()) )
        return true;

    return false;
  }

  /**
   * Tulostaa ne avustuksen lohkot jotka tsmvt hakuehtoon.
   * @param topic mahdolisesti jokereita * ja ? sisltv ehto
   * @return painettiinko poistumista (true) vai ei (false) tulostuksen aikana
   */
  public boolean printMatchingTopics(String topic) {
    printedLineCounter = 0;

    if ( WildChars.containsWildChars(topic) ) {
      for ( Iterator i = topics.keySet().iterator(); i.hasNext() ; ) {
        String s = (String)i.next();
        if ( WildChars.onkoSamat(s, topic) && printTopic(s) )
          return true;
      }
      return false;
    }
    return printTopic(topic);
  }

  /**
   * Selailee avustusta valitun lohkon kohdalta.  Jos lohko == null
   * niin nytetn kohta [SISLLYS]
   * @param topic tulostettava avustuksen kohta
   */
  public void browse(String topic)  {
    while (true) {
      try {
        if (topic == null) topic = INDEX;

        if ( printMatchingTopics(topic) ) return;

        System.out.print("Valitse aihe (voi olla mys *) > ");
        topic = in.readLine();
        if ( topic.length() == 0 ) break;
        if ( quit(topic) ) break;
      }
      catch (IOException ioe) {
        System.out.println(ioe);
      }
    }
  }

  /**
   * Selailee avustusta aloittaen kohdasta [SISLLYS]
   */
  public void browse()  { browse(null); }

  /**
   * Tulostaa topic:in mukaisen lohkon avustuksesta.
   * @param topic tulostetavan lohkon otsikko tai null jolloin selailee avustusta
   */
  public void helpTopic(String topic)  {
    if ( topic == null ) browse();
    else printMatchingTopics(topic);
  }


  public static void main(String[] args) {
    try {
      Help h = new Help("kerho.txt");
      h.helpPrint("Terve!");
      h.helpTopic("Lisys");
      h.helpPrint("#");
      h.browse();
      h.addLine("Uusi otsikko","Eka rivi");
      h.addLine("Uusi otsikko","Toka rivi");
      h.printMatchingTopics("Uusi*");
    }
    catch (IOException ioe) {
      System.err.println(ioe);
    }
  }
}
