package fi.jyu.mit.ohj2;
import java.text.*;

/**
 * Joukko merkkijonoaliohjelmia Ohjelmointi 2-kurssin kyttn
 * @author Vesa Lappalainen
 * @version 1.0, 07.01.2003
 * @version 1.1, 09.03.2002
 * @since 1.4.1
 */

public class Mjonot {
  //****************************************************************************

  /**
   * Palauttaa jonon, jossa on monistettuna jono s n kertaa
   * <pre> tayta("kissa",3)  => "kissakissakissa"</pre>
   * Tuplataan jonon pituus joka kerta => 100 monistusta vaatii 7 kierrosta
   * @param s monistettava jono
   * @param n kuinka monta kertaa monistetaan
   * @return s monistettuna n kertaa
   */
  public static String tayta(String s, int n) {
    int pituus = n*s.length();
    if ( pituus == 0 ) return "";
    StringBuffer jono = new StringBuffer(pituus);
    jono.append(s);
    while ( 2*jono.length() < pituus  ) jono.append(jono);
    jono.append(jono.substring(0,pituus-jono.length()));
    return jono.toString();
  }


  /** Palauttaa merkkijonon jossa on n kappaletta vlilyntej
   * @param n montako vlilynti tarvitaan
   * @return jono, jossa on n kpl vlilyntej
   */
  public static String tyhja(int n) {
    StringBuffer fill = new StringBuffer(n+40);
    while ( fill.length() < n ) fill.append("                                ");
    fill.delete(n,fill.length());
    return fill.toString();
  }

  /**
   * Palauttaa jonon s muotoiltuna vhintn len-pituiseksi
   * <pre>
   * Esim:  fmt("2",3,'0')  => "002"
   *        fmt("2",-3,' ') => "2  "
   * </pre>
   * @param s      muotoiltava jono
   * @param len    pituus, negatiivisella vasempaan laitaan, pos. oikeaan
   * @param fillch merkki jolla tytetn
   * @return       muotoiltu jono
   */
  public static String fmt(String s,int len,char fillch) {
    int needs = Math.abs(len) - s.length();
    if ( needs <= 0 ) return s;
    String fill = tayta(""+fillch,needs);
    if ( len < 0 ) return s + fill;
    return fill + s;
  }

  /**
   * Palauttaa jonon s muotoiltuna vhintn len-pituiseksi
   * <pre>
   * Esim:  fmt("2",3)  => "  2"
   *        fmt("2",-3) => "2  "
   * </pre>
   * @param s     muotoiltava jono
   * @param len   pituus, negatiivisella vasempaan laitaan, pos. oikeaan
   * @return      muotoiltu jono
   */
  public static String fmt(String s,int len) {
    int needs = Math.abs(len) - s.length();
    if ( needs <= 0 ) return s;
    String fill = tyhja(needs);
    if ( len < 0 ) return s + fill;
    return fill + s;
  }

  /**
   * Palauttaa jonon s muotoiltuna vhintn len-pituiseksi
   * @param s     muotoiltava jono
   * @param len   pituus, negatiivisella vasempaan laitaan, pos. oikeaan
   * @return      muotoiltu jono
   * @see #fmt(String s,int len)
   */
  public static String fmt(StringBuffer s,int len) {
    return fmt(s.toString(),len);
  }

  /**
   * Muotoilee kokonaisluvun len kokoiseen tilaan.
   * <pre>fmt(12,4) => "  12";  fmt(12,-4) => "12  "</pre>
   *
   * @param n    muotoiltava luku
   * @param len  tulosjonon vhimmispituus
   * @return     merkkijonoksi muotoiltu luku
   * @see #fmt(String s,int len)
   */
  public static String fmt(int n,int len) {
    return fmt(""+n,len);
  }

  /**
   * Muotoilee kokonaisluvun len kokoiseen tilaan.
   * <pre>fmt(12,4,'0') => "0012";  fmt(12,-4,' ') => "12  "</pre>
   *
   * @param n      muotoiltava luku
   * @param len    tulosjonon vhimmispituus
   * @param fillch merkki jolla tytetn
   * @return       merkkijonoksi muotoiltu luku
   * @see #fmt(String s,int len)
   */
  public static String fmt(int n,int len, char fillch) {
    return fmt(""+n,len,fillch);
  }


  /**
   * Muotoilee reaaliluvun len kokoiseen tilaan.
   * <pre>fmt(12.3,6,2) => "12.30 ";  fmt(12.3,-6,2) => "12.30 "</pre>
   * @param d     muotoiltava luku
   * @param len   tulosjonon vhimmispituus
   * @param desim desimaalien lukumr tulosjonossa
   * @return      merkkijonoksi muotoiltu luku
   * @see #fmt(String s,int len)
   */
  public static String fmt(double d,int len,int desim) {
    String des = "0";
    if ( desim > 0 ) des = "0."+tayta("0",desim);
    DecimalFormat df = new DecimalFormat(des);
    DecimalFormatSymbols ds = new DecimalFormatSymbols();
    ds.setDecimalSeparator('.');
    df.setDecimalFormatSymbols(ds);
    return fmt(df.format(d),len);
  }


  /**
   * Poistetaan jonosta kaikki kaksin- tai useampikertaiset tyhjt.
   * <pre>
   * jono = "   kissa     istuu   ";
   * poista_2_tyhjat(jono)  => jono = " kissa istuu ";
   * </pre>
   * @param s jono josta tyhjt poistetaan
   * @return viite jonoon josta poistettu
   */
  public static StringBuffer poista_2_tyhjat(StringBuffer s) {
    int tyhjia=0;
    int l=0,k=0;
    if ( s == null ) return new StringBuffer("");
    int pit = s.length();

    while ( l < pit ) {
      char c = s.charAt(l);
      if ( c == ' ') tyhjia++;
      else tyhjia = 0;
      if ( tyhjia <= 1 ) s.setCharAt(k++,c);  // Jollei 2-tyhj, kopioidaan!
      l++;
    }
    s.delete(k,pit);
    return s;
  }

  /**
   * Poistetaan jonosta kaikki kaksin- tai useampikertaiset tyhjt.
   * <pre>
   * jono = poista_2_tyhjat("   kissa     istuu   ")  => jono = " kissa istuu ";
   * </pre>
   * @param s jono josta tyhjt poistetaan
   * @return viite uusi jono josta tyhjt poistettu
   */
  public static String poista_2_tyhjat(String s) {
    if ( s == null ) return "";
    return poista_2_tyhjat(new StringBuffer(s)).toString();
  }


  //****************************************************************************
  /**
   * Erottaa jonosta valitun merkin kohdalta alkuosan ja loppuosan.
   * Alkuosa palautetaan funktion nimess ja loppuosa jtetn
   * jonoon.  Merkin etsimissuunta voidaan valita (oletuksena alusta pin).
   * Jos merkki ei lydy, palautetaan koko jono ja tyhjennetn jono.
   * <pre>
   * Kyttesimerkki: olkoon aluksi string jono,s;
   *  1)  jono = "123 456";  s = erota(jono);   => jono == "456"  s == "123"
   *  2)  jono = "123";      s = erota(jono);   => jono == ""     s == "123"
   *  3)  jono = "1 2 3";
   *  while ( jono.length() > 0 ) System.out.print(erota(jono)+",");
   *           => tulostaa 1,2,3,
   * </pre>
   * @param jono           jono jota pilkotaan
   * @param merkki         merkki jonka kohdalta katkaistaan
   * @param etsitakaperin  etsitnk merkki oikealta vasemmalle
   * @return               merkki edeltv jono
   */
  public static String erota(StringBuffer jono, char merkki, boolean etsitakaperin) {
    if ( jono == null ) return "";
    int p;
    if ( !etsitakaperin ) p = jono.indexOf(""+merkki);
    else p = jono.lastIndexOf(""+merkki);
    String alku;
    if ( p < 0 ) {
      alku = jono.toString();
      jono.delete(0,jono.length());
      return alku;
    }
    alku = jono.substring(0,p);
    jono.delete(0,p+1);
    return alku;
  }

  /**
   * Erottaa jonosta valitun merkin kohdalta alkuosan ja loppuosan.
   * @param jono           jono jota pilkotaan
   * @param merkki         merkki jonka kohdalta katkaistaan
   * @return               merkki edeltv jono
   */
  public static String erota(StringBuffer jono, char merkki) {
    return erota(jono,merkki,false);
  }

  /** Erottaa jonosta vlilynnin kohdalta alkuosan ja loppuosan.
   * @param jono jono jota pilkotaan
   * @return     merkki edeltv jono
   */
  public static String erota(StringBuffer jono) {
    return erota(jono,' ',false);
  }

  //****************************************************************************

  public static final String NUMEROT = "0123456789";
  public static final char DESIMAALI = '.';
  public static final String DOUBLENUMEROT = NUMEROT + DESIMAALI;

  //****************************************************************************

  /**
   * Erottaa merkkijonosta seuraavan reaaliluvun.  Jos lukua ei voida erottaa
   * niin palautetaan oletus
   * <pre>
   * jono = " 12.34 km"; d = erotaDouble(jono,0);  => d = 12.34  jono = " km"
   * jono = "12k34 km"; d = erotaDouble(jono,0);  => d = 12  jono = "k34 km"
   * jono = "k12.34 km"; d = erotaDouble(jono,0); => d = 0  jono = "k12.34 km"
   * </pre>
   * @param jono merkkijono josta luku otetaan, jono lyhenee
   * @param oletus arvo jota kytetn jos jonossa ei ole yhtn numeroa
   * @param numerot aluksi sallitut numerot
   * @return otettu reaaliluku
   */
  public static double erotaDouble(StringBuffer jono, double oletus, String numerot) {
    if ( jono == null ) return oletus;
    if ( numerot == null ) numerot = DOUBLENUMEROT;
    int alku = 0; int i; boolean alkuws = true;
    for (i=0; i < jono.length(); i++) {
      char c = jono.charAt(i);
      if ( alkuws && Character.isWhitespace(c) ) { alku++; continue; }
      if ( numerot.indexOf(c) < 0 ) break;
      if ( c == DESIMAALI ) numerot = NUMEROT;
      alkuws = false; // en ei saa ola ws-merkki
    }

    if ( i == 0 ) return oletus;
    String num = jono.substring(alku,i);
    jono.delete(0,i);
    try {
      return Double.parseDouble(num);
    } catch (NumberFormatException ex) {
      return oletus;
    }

  }


  /**
   * Erottaa merkkijonosta seuraavan reaaliluvun.  Jos lukua ei voida erottaa
   * niin palautetaan oletus
   * <pre>
   * jono = " 12.34 km"; d = erotaDouble(jono,0);  => d = 12.34  jono = " km"
   * jono = "12k34 km"; d = erotaDouble(jono,0);  => d = 12  jono = "k34 km"
   * jono = "k12.34 km"; d = erotaDouble(jono,0); => d = 0  jono = "k12.34 km"
   * </pre>
   * @param jono merkkijono josta luku otetaan, jono lyhenee
   * @param oletus arvo jota kytetn jos jonossa ei ole yhtn numeroa
   * @return otettu reaaliluku
   * @see #erotaDouble(StringBuffer jono, double oletus, String numerot)
   */
  public static double erotaDouble(StringBuffer jono, double oletus) {
    return erotaDouble(jono,oletus,DOUBLENUMEROT);
  }


  /**
   * Ottaa merkkijonosta seuraavan kokonaisluvun
   * @param jono merkkijono josta luku otetaan, jono lyhenee
   * @param oletus arvo jota kytetn jos jonossa ei ole yhtn numeroa
   * @return otettu kokonaisluku
   * @see #erotaDouble(StringBuffer jono, double oletus)
   */
  public static int erotaInt(StringBuffer jono, int oletus) {
    return (int)Math.round(erotaDouble(jono,oletus,NUMEROT));
  }

  /**
   * Erottaa merkkijonosta seuraavan reaaliluvun.
   * @param jono merkkijono josta luku otetaan, jono ei muutu
   * @param oletus arvo jota kytetn jos jonossa ei ole yhtn numeroa
   * @return otettu reaaliluku
   * @see #erotaDouble(StringBuffer jono, double oletus)
   */
  public static double erotaDouble(String jono, double oletus) {
    if ( jono == null ) return oletus;
    return erotaDouble(new StringBuffer(jono),oletus);
  }

  /**
   * Ottaa merkkijonosta seuraavan kokonaisluvun
   * @param jono merkkijono josta luku otetaan, jono ei muutu
   * @param oletus arvo jota kytetn jos jonossa ei ole yhtn numeroa
   * @return otettu kokonaisluku
   * @see #erotaDouble(StringBuffer jono, double oletus)
   */
  public static int erotaInt(String jono, int oletus) {
    if ( jono == null ) return oletus;
    return erotaInt(new StringBuffer(jono),oletus);
  }

  /**
   * Ottaa merkkijonosta seuraavan merkin
   * @param jono merkkijono josta merkki otetaan
   * @param oletus arvo jota kytetn jos jonossa ei ole yhtn merkki
   * @return otettu merkki
   */
  public static char erotaChar(StringBuffer jono, char oletus) {
    if ( jono == null ) return oletus;
    if ( jono.length() == 0 ) return oletus;
    char c = jono.charAt(0);
    jono.delete(0,1);
    return c;
  }

  /**
   * Erottaa jonosta valitun merkin kohdalta alkuosan ja loppuosan.
   * Jono trimmataan ennen palautusta (kaikki tyhjt pois)
   * Kytt oletusta jos jono on tyhj.
   * Tehty lhinn "kaveriksi" vastaaville int ja double funktiolle,
   * jotta funktiopolymorfismin avulla saadaan symmetrisi aliohjelmia
   * jolloin kutsussa on helppo vaihtaa parametrin tyyppi
   * <pre>
   *   String s = erota(jono,'|',"");
   *   double d = erota(jono,'|',0.0);
   *   s = erota(jono,'|',s);
   *   d = erota(jono,'|',d);
   * </pre>
   * @param jono           jono jota pilkotaan
   * @param merkki         merkki jonka kohdalta katkaistaan
   * @param oletus         arvo jota kytetn jos palasesta tulee tyhj
   * @return               merkki edeltv jono
   * @see #erota(StringBuffer jono, char merkki, boolean etsitakaperin)
   */
  public static String erota(StringBuffer jono, char merkki, String oletus) {
    String pala = erota(jono,merkki,false);
    if ( oletus == null ) oletus = "";
    if ( pala.equals("") ) pala = oletus;
    return poista_2_tyhjat(pala.trim());
  }

  /**
   * Ottaa merkkijonosta ensin seuraavan palasen merkki-parametriin
   * saakka ja sitten ottaa tst palasta reaaliluvun
   * @param jono           jono jota pilkotaan
   * @param merkki         merkki jonka kohdalta katkaistaan
   * @param oletus         arvo jota kytetn jos palasta ei saa reaalilukua
   * @return               merkki edeltv jono
   * @see #erota(StringBuffer jono, char merkki, String oletus)
   */
  public static double erota(StringBuffer jono, char merkki, double oletus) {
    String pala = erota(jono,merkki,false);
    return erotaDouble(pala,oletus);
  }


  /**
   * Ottaa merkkijonosta ensin seuraavan palasen merkki-parametriin
   * saakka ja sitten ottaa tst palasta kokonaisluvun
   * @param jono           jono jota pilkotaan
   * @param merkki         merkki jonka kohdalta katkaistaan
   * @param oletus         arvo jota kytetn jos palasta ei saa reaalilukua
   * @return               merkki edeltv jono
   * @see #erota(StringBuffer jono, char merkki, String oletus)
   */
  public static int erota(StringBuffer jono, char merkki, int oletus) {
    String pala = erota(jono,merkki,false);
    return erotaInt(pala,oletus);
  }


  /**
   * Erottaa jonosta valitun merkin kohdalta alkuosan ja loppuosan.
   * Jono trimmataan ennen palautusta (kaikki tyhjt pois)
   * Heitt poikkeuksen jos jono on tyhj
   * Tehty lhinn "kaveriksi" vastaaville int ja double funktiolle,
   * jotta funktiopolymorfismin avulla saadaan symmetrisi aliohjelmia
   * jolloin kutsussa on helppo vaihtaa parametrin tyyppi
   * <pre>
   *   String s = null;
   *   double d = 0.0;
   *   try {
   *     s = erotaEx(jono,'|',s);
   *     d = erotaEx(jono,'|',d);
   *   } catch ( NumberFormatException ex ) {
   *     System.out.println(ex.getMessage());
   *   }
   * </pre>
   * @param jono           jono jota pilkotaan
   * @param merkki         merkki jonka kohdalta katkaistaan
   * @param oletus         arvo mikli tulos olisi tyhj
   * @return               merkki edeltv jono
   * throws NumberFormatException jos palaa ei saada
   * @see #erota(StringBuffer jono, char merkki, boolean etsitakaperin)
   */
  public static String erotaEx(StringBuffer jono, char merkki, String oletus)
    throws NumberFormatException {
    if ( (jono == null) || jono.equals("") )
      throw new NumberFormatException("jono = \"\"");
    String pala = erota(jono,merkki,false);
    if ( oletus == null ) oletus = "";
    if ( pala.equals("") ) pala = oletus;
    return poista_2_tyhjat(pala.trim());
  }

  /**
   * Ottaa merkkijonosta ensin seuraavan palasen merkki-parametriin
   * saakka ja sitten ottaa tst palasta reaaliluvun.
   * @param jono           jono jota pilkotaan
   * @param merkki         merkki jonka kohdalta katkaistaan
   * @param oletus         tuo vain tiedon palautettavasta tyypist
   * @return               merkki edeltv jono
   * throws NumberFormatException jos reaalilukua ei saada
   * @see #erotaEx(StringBuffer jono, char merkki, String oletus)
   */
  public static double erotaEx(StringBuffer jono, char merkki, double oletus)
    throws NumberFormatException {
    String pala = erota(jono,merkki,false);
    double d = erotaDouble(pala,Double.MIN_VALUE);
    if ( d == Double.MIN_VALUE )
      throw new NumberFormatException("jono = \"" + pala + "\"");
    return d;
  }


  /**
   * Ottaa merkkijonosta ensin seuraavan palasen merkki-parametriin
   * saakka ja sitten ottaa tst palasta kokonaisluvun
   * @param jono           jono jota pilkotaan
   * @param merkki         merkki jonka kohdalta katkaistaan
   * @param oletus         tuo vain tiedon palautettavasta tyypist
   * @return               merkki edeltv jono
   * throws NumberFormatException jos kokonaislukua ei saada
   * @see #erotaEx(StringBuffer jono, char merkki, String oletus)
   */
  public static int erotaEx(StringBuffer jono, char merkki, int oletus)
    throws NumberFormatException {
    String pala = erota(jono,merkki,false);
    int i = erotaInt(pala,Integer.MIN_VALUE);
    if ( i == Integer.MIN_VALUE )
      throw new NumberFormatException("jono = \"" + pala + "\"");
    return i;
  }



  //****************************************************************************


  private static void tulostaViiva(String s) {
    System.out.println("");
    System.out.println("=====================================================");
    System.out.println(s);
    System.out.println("=====================================================");
  }

  public static void main(String[] args) {
    tulostaViiva("Testataan erota-metodia.");
    StringBuffer jono = new StringBuffer("1 2 3");
    while ( jono.length() > 0 ) {
      System.out.print(erota(jono)+",");
    }
    System.out.println("");

    tulostaViiva("Testataan erotaDouble ja erotaInt-metodeja");
    String testi[] = {" 12.34 km", "12k34", "k12.34", "12.34.56", ".34 km" };
    for ( int i = 0; i < testi.length; i++ ) {
      StringBuffer jono2 = new StringBuffer(testi[i]);
      double d = erotaDouble(jono2,0);
      System.out.println(fmt("\""+testi[i] + "\"",-15) + fmt(" d=" + d,-10) +
                             " jono = \"" + jono2+ "\"");
    }
    for ( int i = 0; i < testi.length; i++ ) {
      StringBuffer jono2 = new StringBuffer(testi[i]);
      int d = erotaInt(jono2,99);
      System.out.println(fmt("\""+testi[i] + "\"",-15) + fmt(" i=" + d,-10) +
                             " jono = \"" + jono2+ "\"");
    }

    tulostaViiva("Testataan tayta-metodia");
    for (int i=0; i<17; i++)
      System.out.println(tayta("kissa",i));

    tulostaViiva("Testataan fmt-metodia");
    for (int i=3; i<10; i++) System.out.println(fmt(1.23456,i,3));
    for (int i=3; i<7; i++) System.out.println(fmt(-1.23456,i,0));
    for (int i=9; i<14; i++) System.out.println(fmt(1234567.89,i,3));

    tulostaViiva("Testataan polymorfista erota-metodia");
    StringBuffer jono3 = new StringBuffer(" Volvo    145 |  12300 e | 1 kpl");
    String s=""; double d=0.0; int i=0;
    s = erota(jono3,'|',s);
    d = erota(jono3,'|',d);
    i = erota(jono3,'|',i);
    System.out.println("'"+s+"'  " + fmt(d,4,2) + "   " + i);

    tulostaViiva("Testataan polymorfista erotaEx-metodia");
    StringBuffer jono4 = new StringBuffer(" 23.4 | 12 | kala | 99");
    d = 0;

    for (i=0; i<5; i++ ) {
      try {
        d = erotaEx(jono4,'|',d);
      } catch (NumberFormatException ex) {
        System.out.println("Virhe: " + ex.getMessage());
      }
      System.out.println("d = " + fmt(d,4,2) + "  jono = \"" + jono4 + "\"");
    }


  }
}
