package fi.jyu.mit.ohj2;
import java.io.*;

/**
 * <p>Title: Syotto</p>
 * <p>Description: Aliohjelmia tietojen lukemiseen ptteelt</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: jyu </p>
 * @author Vesa Lappalainen
 * @version 1.0/08.01.2003
 * @version 1.1/25.01.2003
 */

public class Syotto {

  /**
   * Aliohjelmalla kysytn kyttjlt arvo merkkijonolle.
   * Jos tulOletus on tosi, tulostetaan oletusarvo.
   * Jos kyttj painaa pelkstn ret, palautetaan oletusarvo
   *
   * @param kysymys kyttjlle tulostettava kysymys
   * @param oletus arvo jota kytetn jos painetaan pelkk ret
   * @param tulOletus tulostetaanko oletus kyttjlle
   * @return kyttjn syttm merkkijono
   */
  public static String kysy(String kysymys, String oletus, boolean tulOletus) {
    BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
    while ( true ) {
      System.out.print(kysymys);
      if ( tulOletus ) System.out.print(" (" + oletus + ")");
      System.out.print(" >");
      String s = "";
      try {
        s = in.readLine();
      } catch (IOException ex) {
        continue;
      }
      if ( ( s == null ) || ( s.equals("") ) ) return oletus;
      return s;
    }

  }

  /**
   * Aliohjelmalla kysytn kyttjlt arvo merkkijonolle.
   * Tulostetaan oletusarvo.
   * Jos kyttj painaa pelkstn ret, palautetaan oletusarvo
   *
   * @param kysymys kyttjlle tulostettava kysymys
   * @param oletus arvo jota kytetn jos painetaan pelkk ret
   * @return kyttjn syttm merkkijono
   */
  public static String kysy(String kysymys, String oletus) {
    return kysy(kysymys,oletus,true);
  }

  /**
   * Aliohjelmalla kysytn kyttjlt arvo merkkijonolle.
   * Ei tulosteta oletusarvoa.
   *
   * @param kysymys kyttjlle tulostettava kysymys
   * @return kyttjn syttm merkkijono
   */
  public static String kysy(String kysymys) {
    return kysy(kysymys,"",false);
  }

  /**
   * Kysytn reaaliluku. Jos annetaan ei-luku, kysytn uudelleen.
   * @param kysymys nytlle tulostettava kysymys
   * @param oletus arvo jota kytetn jos painetaanpelkk Ret
   * @param tulOletus tulostetaanko oletus kyttjlle
   * @return kyttjn kirjoittama reaaliluku
   */
  public static double kysy(String kysymys, double oletus, boolean tulOletus) {
    while ( true ) {
      String s = kysy(kysymys,""+oletus,tulOletus);
      s = s.trim();
      if ( s.length() == 0 ) return oletus;
      try {
        return Double.parseDouble(s);
      } catch (NumberFormatException ex) {
        System.out.println("Ei numero: \"" + s + "\"");
      }
    }
  }

  /**
   * Kysytn reaaliluku. Jos annetaan ei-luku, kysytn uudelleen.
   * Oletusarvo tulostetaan sulkuihin.
   * @param kysymys nytlle tulostettava kysymys
   * @param oletus arvo jota kytetn jos painetaanpelkk Ret
   * @return kyttjn kirjoittama reaaliluku
   */
  public static double kysy(String kysymys, double oletus) {
    return kysy(kysymys,oletus,true);
  }

  /**
   * Kysytn reaaliluku. Jos annetaan ei-luku, kysytn uudelleen.
   * Ei tulosteta oletusta.
   * @param kysymys nytlle tulostettava kysymys
   * @return kyttjn kirjoittama reaaliluku
   */
  public static double kysyDouble(String kysymys) {
    return kysy(kysymys,0.0,false);
  }

  /**
   * Kysytn kokonaisluku. Jos annetaan ei-luku, kysytn uudelleen.
   * @param kysymys nytlle tulostettava kysymys
   * @param oletus arvo jota kytetn jos painetaanpelkk Ret
   * @param tulOletus tulostetaanko oletus kyttjlle
   * @return kyttjn kirjoittama kokonaisluku
   */
  public static int kysy(String kysymys, int oletus, boolean tulOletus) {
    while ( true ) {
      String s = kysy(kysymys,""+oletus,tulOletus);
      s = s.trim();
      if ( s.length() == 0 ) return oletus;
      try {
        return Integer.parseInt(s);
      } catch (NumberFormatException ex) {
        System.out.println("Ei kokonaisluku: \"" + s + "\"");
      }
    }
  }

  /**
   * Kysytn kokonaisluku. Jos annetaan ei-luku, kysytn uudelleen.
   * Oletusarvo tulostetaan sulkuihin.
   * @param kysymys nytlle tulostettava kysymys
   * @param oletus arvo jota kytetn jos painetaanpelkk Ret
   * @return kyttjn kirjoittama kokonaisluku
   */
  public static int kysy(String kysymys, int oletus) {
    return kysy(kysymys,oletus,true);
  }

  /**
   * Kysytn kokonaisluku. Jos annetaan ei-luku, kysytn uudelleen.
   * Ei tulosteta oletusta.
   * @param kysymys nytlle tulostettava kysymys
   * @return kyttjn kirjoittama kokonaisluku
   */
  public static int kysyInt(String kysymys) {
    return kysy(kysymys,0,false);
  }


  public static void main(String[] args) {
    String s = kysy("Anna jono","Kissa");
    System.out.println("Jono oli: \"" + s + "\"");
    String s2 = kysy("Anna jono");
    System.out.println("Jono oli: \"" + s2 + "\"");
    double d = kysy("Anna reaaliluku",3.14);
    System.out.println("Luku oli: \"" + d + "\"");
    int i = kysyInt("Anna kokonaisluku");
    System.out.println("Luku oli: \"" + i + "\"");
    d = kysyDouble("Anna reaaliluku");
    System.out.println("Luku oli: \"" + d + "\"");
  }
}
