package fi.jyu.mit.ohj2;
import java.io.*;

/**
 * Luokka wildmat-vertailuun, jossa jokereita ovat * ja ?
 * @author Vesa Lappalainen, Markku Vire
 * @version 1.0, 23.03.2002
 */
public class WildChars {

  /**
   * Palauttaa sisltk jono str jokerimekrkkej * tai ?
   * @param str jono jota tutkitaan
   * @return sisltk jono (true) jokerimerkkej vai ei (false)
   */
  public static boolean containsWildChars(String str)
  {
    if (str == null)  return false;
    return (str.indexOf('*') >= 0 || str.indexOf('?') >= 0);
  }

  /**
   * Apumetodi wildmat-aliohjelman toteuttamiseksi
   * @param str  tutkittava jono
   * @param mask maski johon verrataan
   * @param si   indeksi jonossa, josta aloitetaan vertailu
   * @param mi   indeksi maskissa, josta aloitetaan vertailu
   * @return ovatko samat (true) vai ei (false) jokeri-mieless
   */
  private static boolean wildmat(String str, String mask, int si, int mi) {
    if ( str == null || mask == null ) return ( str == null && mask == null );

    while ( mi < mask.length() ) {
      char m = mask.charAt(mi);

      if ( m == '?' ) {
        if (si >= str.length()) return false;
      }
      else if ( m == '*' ) {
        mi++;
        if ( mi < mask.length() )
          while ( !wildmat(str, mask,si,mi) ) {
            si++;
            if ( si >= str.length() ) return false;
          }
        return true;
      }
      else if ( ( si >= str.length() ) || ( m != str.charAt(si) ) )
        return false;

      si++; mi++;
    }

    return ( si == str.length() );
  }

  /**
   *<pre>
   *  Funktiolla tutkitaan tsmk annettu jono verrattavaan maskiin.
   * Maski saa sislt seuraavia erikoismerkkej:
   *   * vastaa 0-n merkki
   *   ? vastaa mit tahansa yht merkki
   *
   * Algoritmi: Kysymysmerkki ja tavallinen kirjain normaalisti
   *            Jos tulee vastaan thti joka ei ole jonon lopussa,
   *            niin ongelmahan on oikeastaan
   *            (koska thn asti kaikki on ollut oikein)
   *            "Onko loppujono sama kuin toisen jonon loppu JOSTAKIN
   *             kohdasta alkaen"?
   *            Siis kokeillaan sovittaa loppujonoa aliohjelman itsens
   *            (rekursio) avulla kaikkiin mahdollisiin loppupaikkoihin.
   * Esimerkki: str = "Kissa" maski = "*ss*" -> true
   *                                = "*ss"  -> false
   * </pre>
   *
   * @param str   jono jota tutkitaan
   * @param mask  maski johon verrataan
   * @return      onko samat (true) vai ei (false) maskin mieless.
   */
  public static boolean wildmat(String str, String mask) {
    return wildmat(str,mask,0,0);
  }


  public static boolean onkoSamat(String str, String mask) {
    return wildmat(str.toUpperCase(),mask.toUpperCase());
  }


  static int wilderr = 0;

  public static  void wildtest(String s, String maski, boolean result) {
    boolean ret = onkoSamat(s,maski);
    System.out.print("\"" + s + "\"  \"" + maski + "\"  => " + ret);
    if ( ret != result ) {
      System.out.print("  VRIN!");
      wilderr++;
    }
    System.out.println();
  }

  public static void main(String[] args) {
    wildtest("kissa","kissa",true);
    wildtest("kissa","kiss*",true);
    wildtest("kissa","kss*",false);
    wildtest("kissa","k*ss*",true);
    wildtest("kissa","k***********ss*",true);
    wildtest("kissa","*iss*",true);
    wildtest("kissa","*kiss*",true);
    wildtest("kissa","*kissa*",true);
    wildtest("kissa","*k?ss*",true);
    wildtest("kissa","kass*",false);
    wildtest("","*a",false);
    wildtest("","*",true);
    wildtest("","",true);
    wildtest("a","",false);
    wildtest("kissa","KISSA",true);
    if ( wilderr == 0 ) System.out.println("Kaikki oikein :-)");
    else  System.out.println(" VIRHEIT: " + wilderr);
  }
}
